// 005 u_c_move_command_18
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_move_command_18;
  interface
    uses Windows;

    type c_graphic= class
                      Constructor create_graphic;
                      procedure move(p_delta: tPoint);
                      Destructor Destroy; Override;
                    end; // c_graphic

         c_constraint_solver_memento= Class;

         c_move_command= Class
                           _m_c_state: c_constraint_solver_memento;
                           _m_delta: tPoint;
                           _m_c_graphic_target: c_graphic;

                           constructor create_move_command(p_c_graphic_target: c_graphic;
                               p_delta: tPoint);
                           procedure execute;
                           procedure undo;
                           Destructor Destroy; Override;
                         end; // c_move_command

         c_constraint_solver= Class
                                // -- this is a singleton class
                                // -- here non-trivial state and handling of connection

                                constructor create_constraint_solver;

                                // -- for the singleton handling:
                                class function f_c_constraint_solver_instance: c_constraint_solver;

                                procedure solve;
                                procedure add_constraint(p_c_start_connection_graphic, p_c_end_connection_graphic: c_graphic);
                                procedure remove_constraint(p_c_start_connection_graphic, p_c_end_connection_graphic: c_graphic);
                                function f_c_create_constraint_solver_memento: c_constraint_solver_memento;
                                procedure set_memento(p_c_constraint_solver_memento: c_constraint_solver_memento);
                              end; // c_constraint_solver

         c_constraint_solver_memento= Class
                                        // -- todo friend c_constraint_solver
                                        constructor create_constraint_solver_memento;
                                        Destructor Destroy; Override;
                                      end; // c_constraint_solver_memento
  implementation

    // -- c_graphic

    Constructor c_graphic.create_graphic;
      begin
        Inherited Create;
      end; // create_graphic

    procedure c_graphic.move(p_delta: tPoint);
      begin
      end; // move

    Destructor c_graphic.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_move_command

    constructor c_move_command.create_move_command(p_c_graphic_target: c_graphic;
        p_delta: tPoint);
      begin
      end; // create_move_command

    procedure c_move_command.execute;
      var l_c_constraint_solver: c_constraint_solver;
          l_c_state: c_constraint_solver_memento;
      begin
        // -- get the singleton
        l_c_constraint_solver:= c_constraint_solver.f_c_constraint_solver_instance;
        // -- acquire a memento before the move
        _m_c_state:= l_c_constraint_solver.f_c_create_constraint_solver_memento;
        _m_c_graphic_target.move(_m_delta);
        // -- now sove the constraint
        l_c_constraint_solver.solve;
      end; // execute

    procedure c_move_command.undo;
      var l_c_constraint_solver: c_constraint_solver;
          l_c_state: c_constraint_solver_memento;
      begin
        // -- get the singleton back
        l_c_constraint_solver:= c_constraint_solver.f_c_constraint_solver_instance;
        // -- reverse the move
        _m_c_graphic_target.move(_m_delta);
        // -- restore the solver state
        l_c_constraint_solver.set_memento(_m_c_state);
        // -- now sove the constraint
        l_c_constraint_solver.solve;
      end; // undo

    Destructor c_move_command.Destroy;
      begin
      end; // Destroy

    // -- c_constraint_solver

    constructor c_constraint_solver.create_constraint_solver;
      begin
      end; // create_constraint_solver

    var // -- the singleton
        _g_c_constraint_solver_instance: c_constraint_solver= Nil;

    class function c_constraint_solver.f_c_constraint_solver_instance: c_constraint_solver;
      begin
        if _g_c_constraint_solver_instance= Nil
          then _g_c_constraint_solver_instance:= c_constraint_solver.create_constraint_solver;
        Result:= _g_c_constraint_solver_instance;
      end; // f_c_constraint_solver_instance

    procedure c_constraint_solver.solve;
      begin
      end; // solve

    procedure c_constraint_solver.add_constraint(p_c_start_connection_graphic, p_c_end_connection_graphic: c_graphic);
      begin
      end; // add_constraint

    procedure c_constraint_solver.remove_constraint(p_c_start_connection_graphic, p_c_end_connection_graphic: c_graphic);
      begin
      end; // remove_constraint

    function c_constraint_solver.f_c_create_constraint_solver_memento: c_constraint_solver_memento;
      begin
      end; // f_c_create_constraint_solver_memento

    procedure c_constraint_solver.set_memento(p_c_constraint_solver_memento: c_constraint_solver_memento);
      begin
      end; // set_memento

    // -- c_constraint_solver_memento

    constructor c_constraint_solver_memento.create_constraint_solver_memento;
      begin
      end; // create_constraint_solver_memento

    Destructor c_constraint_solver_memento.Destroy;
      begin
      end; // Destroy







    begin // u_c_graphic
    end. // u_c_graphic
